/*
 * Copyright (c) 2011 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 16-05-2011
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package eu.unicore.ucc.ccm;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.apache.commons.io.FileUtils;

import de.fzj.unicore.ucc.Constants;
import de.fzj.unicore.ucc.MessageWriter;

import xmlbeans.eu.unicore.ccm.internal.ClientProfileDocument.ClientProfile;
import xmlbeans.eu.unicore.ccm.internal.RegistryType;

/**
 * Updates configuration in properties format. Currently only registries are
 * supported.
 * @author K. Benedyczak
 */
public class PropertiesUpdater
{
	public static String DEFAULT_HOST = "localhost";
	public static String DEFAULT_VSITE = "DEMO-SITE";
	
	private MessageWriter parent;
	private String workingDir;
	private boolean removeDefault;
	
	public PropertiesUpdater(MessageWriter reporter, String workingDir)
	{
		this.parent = reporter;
		this.workingDir = workingDir;
	}
	
	public void processRegistries(ClientProfile profile, String propertiesFile, 
			boolean update) throws CCMException
	{
		RegistryType registries[] = profile.getRegistryArray();
		if (registries == null || registries.length == 0)
		{
			parent.verbose("No registries defined in the configuration profile, " +
					"skipping an update of properties.");
			return;
		}
		removeDefault = profile.getRemoveDefaults();
		
		File propsFile = new File(propertiesFile);
		if (!propsFile.exists())
		{
			parent.message("Properties file " + propertiesFile + " does not exist. " +
					"It will be created.");
			try
			{
				File parent = propsFile.getParentFile();
				if (parent != null)
					parent.mkdirs();
				propsFile.createNewFile();
			} catch (IOException e)
			{
				throw new CCMException("Can't create an empty properties file", e);
			}
		}
		
		if (update)
			CCMAction.backupFile(propertiesFile, parent);
		else
		{
			try
			{
				FileUtils.copyFileToDirectory(new File(propertiesFile), 
						new File(workingDir));
				propertiesFile = workingDir + File.separator + 
					new File(propertiesFile).getName();
			} catch (IOException e)
			{
				throw new CCMException("Can't copy the " +
						"original configuration", e);
			}
		}
		
		try
		{
			processPropertiesFile(propertiesFile, registries);
		} catch (ConfigurationException e)
		{
			throw new CCMException("Can't update the " +
					" configuration", e);
		} catch (MalformedURLException e)
		{
			throw new CCMException("Can't update the configuration as " +
					"one of registry URLs was invalid", e);
		} catch (IOException e)
		{
			throw new CCMException("Can't update the configuration as " +
					"there is a I/O error", e);
		}
	}
	
	private void processPropertiesFile(String file, RegistryType []registries) 
		throws ConfigurationException, IOException
	{
		Map<URL, RegistryType> newRegs = new LinkedHashMap<URL, RegistryType>();
		for (RegistryType reg: registries)
		{
			URL u = new URL(reg.getAddress());
			newRegs.put(u, reg);
		}
	
		PropertiesFileHelper propsHelper = new PropertiesFileHelper(file);
		PropertiesConfiguration config = propsHelper.getPropertiesConfiguration();
		
		String registryURL = config.getString(Constants.OPT_REGISTRY_LONG);
		String[] urls = registryURL != null ? registryURL.split("[, ]") : new String[0];
		
		StringBuilder newRegistries = new StringBuilder();
		boolean first = true;
		for (String existing: urls)
		{
			if (existing.trim().equals(""))
				continue;
			URL rURL = new URL(existing);
			if (removeDefault)
			{
				if (rURL.getHost().equals(DEFAULT_HOST) && 
						rURL.getPath().contains(DEFAULT_VSITE))
				{
					parent.verbose("Removing the default registry: " + existing);
					continue;
				}
			}
			if (newRegs.containsKey(rURL))
			{
				parent.verbose("Registry " + rURL + " already present.");
				newRegs.remove(rURL);
			}
			if (!first)
				newRegistries.append(" ");				
			newRegistries.append(existing);
			first = false;
		}
		
		Iterator<URL> newOnes = newRegs.keySet().iterator();
		while (newOnes.hasNext())
		{
			if (!first)
				newRegistries.append(" ");
			String toAdd = newOnes.next().toString();
			newRegistries.append(toAdd);
			first = false;
			parent.message("Added a new registry: " + toAdd);
		}
		
		config.setProperty(Constants.OPT_REGISTRY_LONG, newRegistries.toString());
		
		propsHelper.close();
	}	
}











