/*
 * Copyright (c) 2011 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 16-05-2011
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package eu.unicore.ucc.ccm;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.cert.X509Certificate;
import java.util.List;

import org.apache.commons.io.FileUtils;

import de.fzj.unicore.ucc.MessageWriter;

import xmlbeans.eu.unicore.ccm.internal.TrustedCAType;
import xmlbeans.eu.unicore.ccm.internal.ClientProfileDocument.ClientProfile;
import eu.unicore.ccm.ProfileHandler;

/**
 * Updates the trusted CAs
 * @author K. Benedyczak
 */
public class TrustUpdater
{
	public static final String DEFAULT_CA = "demo ca for testing only";
	private char[] password;
	private String type;
	private String truststore;
	private MessageWriter parent;
	private String workingDir;
	
	public TrustUpdater(MessageWriter parent, String workingDir)
	{
		this.parent = parent;
		this.workingDir = workingDir;
	}
	
	public void processTrustedCAs(ClientProfile profile, KeyStoreParameters tsParams, 
			boolean update) throws CCMException
	{
		List<X509Certificate> certs = fetchTrustedCAs(profile);
		password = tsParams.getPasswd();
		type = tsParams.getType();
		truststore = tsParams.getPath();
		
		if (update)
			CCMAction.backupFile(truststore, parent);
		else
		{
			try
			{
				FileUtils.copyFileToDirectory(new File(truststore), 
					new File(workingDir));
				truststore = workingDir + File.separator + 
					new File(truststore).getName();
			} catch (IOException e)
			{
				throw new CCMException("Can't copy the " +
						"original truststore", e);
			}
		}
		KeyStore ts = loadTS();
		addTrustedCAs(certs, profile, ts);
		saveTS(ts);
	}
	
	
	private KeyStore loadTS() throws CCMException
	{
		try
		{
			InputStream is = new BufferedInputStream(new FileInputStream(
					truststore));
			KeyStore ret = KeyStore.getInstance(type);
			ret.load(is, password);
			is.close();
			return ret;
		} catch (GeneralSecurityException e)
		{
			throw new CCMException("Can't load the truststore " + truststore, e);
		} catch (IOException e)
		{
			throw new CCMException("Can't load the truststore " + truststore, e);
		}
	}
	
	private void saveTS(KeyStore ts) throws CCMException
	{
		OutputStream os;
		try
		{
			os = new BufferedOutputStream(new FileOutputStream(truststore));
			ts.store(os, password);
			os.close();
		} catch (GeneralSecurityException e)
		{
			throw new CCMException("Can't save the truststore " + truststore, e);
		} catch (IOException e)
		{
			throw new CCMException("Can't save the truststore " + truststore, e);
		}
	}
	
	private List<X509Certificate> fetchTrustedCAs(ClientProfile profile) throws CCMException
	{
		ProfileHandler handler = new ProfileHandler(profile);
		List<X509Certificate> certs;
		try
		{
			certs = handler.downloadCACerts();
		} catch (IOException e)
		{
			throw new CCMException("Can't download trusted CA certificates" +
					" referenced in the profile", e);
		}
		return certs;
	}
	
	private void addTrustedCAs(List<X509Certificate> certs, ClientProfile profile, 
			KeyStore ks) throws CCMException
	{
		if (profile.getRemoveDefaults())
		{
			try
			{
				ks.deleteEntry(DEFAULT_CA);
			} catch (KeyStoreException e)
			{
				parent.error("Can't remove the default trusted CA", e);
			}
		}
		TrustedCAType []trustedCAMeta = profile.getTrustedCAArray();
		for (int i=0; i<certs.size(); i++)
		{
			String alias = trustedCAMeta[i].getSuggestedLabel();
			X509Certificate cert = certs.get(i);
			try
			{
				if (ks.getCertificateAlias(cert) != null)
				{
					parent.verbose("Skipping trusted certificate " + 
							cert.getSubjectX500Principal() + 
							" as it is already present in the truststore.");
					continue;
				}
				if (ks.containsAlias(alias))
				{
					int j=1;
					while(ks.containsAlias(alias + "_" + j))
						j++;
					alias = alias + "_" + j;
				}
				parent.verbose("Will add CA " + cert.getSubjectX500Principal() + 
						" under " + alias + " alias.");
				ks.setCertificateEntry(alias, cert);
				parent.message("Added a new trusted CA: " + cert.getSubjectX500Principal());
			} catch (Exception e)
			{
				throw new CCMException("Can't add trusted CA " +
						"certificate to the keystore", e);
			}
		}
		
	}
}
