/*
 * Copyright (c) 2011 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 27-05-2011
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package eu.unicore.ucc.ccm;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.security.KeyStore;
import java.security.cert.X509Certificate;
import java.util.Enumeration;

import javax.security.auth.x500.X500Principal;

import junit.framework.Assert;

import org.apache.commons.io.FileUtils;
import org.junit.Test;

import xmlbeans.eu.unicore.ccm.internal.TrustedCAType;
import xmlbeans.eu.unicore.ccm.internal.ClientProfileDocument.ClientProfile;

public class TrustUpdaterTest
{
	@Test
	public void testNewWithDefault1()
	{
		KeyStoreParameters ksParams = new KeyStoreParameters("src/test/resources/keystore1.jks", 
				"JKS", "the!user".toCharArray());
		ClientProfile profile = ClientProfile.Factory.newInstance();
		TrustedCAType tr1 = profile.addNewTrustedCA();
		tr1.setCertificateURL("file:src/test/resources/ca1.pem");
		tr1.setSuggestedLabel("ca1");
		
		String file = performUpdate(ksParams, profile, false);
		testKS(new KeyStoreParameters(file, ksParams.getType(), ksParams.getPasswd()),
				"CN=PDPTest Server,O=Testing Organization,L=Testing City,C=EU", "ca1", 2);
		testKS(new KeyStoreParameters(file, ksParams.getType(), ksParams.getPasswd()),
				"OU=Testing, O=UNICORE.EU, CN=Demo CA for testing only, C=DE", "demo ca for testing only", 2);
		FileUtils.deleteQuietly(new File("target/test-workdir"));
	}

	@Test
	public void testNewRmDefault1()
	{
		KeyStoreParameters ksParams = new KeyStoreParameters("src/test/resources/keystore1.jks", 
				"JKS", "the!user".toCharArray());
		ClientProfile profile = ClientProfile.Factory.newInstance();
		profile.setRemoveDefaults(true);
		TrustedCAType tr1 = profile.addNewTrustedCA();
		tr1.setCertificateURL("file:src/test/resources/ca1.pem");
		tr1.setSuggestedLabel("ca1");
		
		String file = performUpdate(ksParams, profile, false);
		testKS(new KeyStoreParameters(file, ksParams.getType(), ksParams.getPasswd()),
				"CN=PDPTest Server,O=Testing Organization,L=Testing City,C=EU", "ca1", 1);
		FileUtils.deleteQuietly(new File("target/test-workdir"));
	}

	@Test
	public void testUpdateRmDefault1()
	{
		KeyStoreParameters ksParams = new KeyStoreParameters("src/test/resources/keystore1.jks", 
				"JKS", "the!user".toCharArray());
		ClientProfile profile = ClientProfile.Factory.newInstance();
		profile.setRemoveDefaults(true);
		TrustedCAType tr1 = profile.addNewTrustedCA();
		tr1.setCertificateURL("file:src/test/resources/ca1.pem");
		tr1.setSuggestedLabel("ca1");
		
		String file = performUpdate(ksParams, profile, true);
		testKS(new KeyStoreParameters(file, ksParams.getType(), ksParams.getPasswd()),
				"CN=PDPTest Server,O=Testing Organization,L=Testing City,C=EU", "ca1", 1);
		FileUtils.deleteQuietly(new File("target/test-workdir"));
	}

	private String performUpdate(KeyStoreParameters params, ClientProfile profile, boolean update)
	{
		File workdir = new File("target/test-workdir");
		workdir.mkdir();
		try
		{
			FileUtils.copyFileToDirectory(new File(params.getPath()), workdir);
		} catch (IOException e)
		{
			e.printStackTrace();
			Assert.fail(e.toString());
		}
		File newDir;
		if (!update)
		{
			newDir = new File(workdir, "new-configs");
			newDir.mkdir();
		} else
			newDir = workdir;
		
		TrustUpdater updater = new TrustUpdater(
				new MockMessageWriter(), 
				newDir.getAbsolutePath());
		
		File workFile = new File(workdir, new File(params.getPath()).getName());
		params.setPath(workFile.getAbsolutePath());
		try
		{
			updater.processTrustedCAs(profile, params, update);
		} catch (CCMException e)
		{
			e.printStackTrace();
			Assert.fail(e.toString());
		}
		if (update)
			return workFile.toString();
		else
			return new File(newDir, new File(params.getPath()).getName()).toString();
	}

	private void testKS(KeyStoreParameters ksParams, String reqTrustedDN, String labelPrefix, int casNum)
	{
		try
		{
			KeyStore ks = KeyStore.getInstance(ksParams.getType());
			ks.load(new FileInputStream(ksParams.getPath()), ksParams.getPasswd());
			Enumeration<String> aliases = ks.aliases();
			int num = 0;
			boolean found = false;
			while (aliases.hasMoreElements())
			{
				String alias = aliases.nextElement();
				if (!ks.isCertificateEntry(alias))
					continue;
				X509Certificate cert = (X509Certificate)ks.getCertificate(alias);
				num++;
				if (cert.getSubjectX500Principal().equals(new X500Principal(reqTrustedDN)))
				{
					if (alias.startsWith(labelPrefix))
					{
						found = true;
						continue;
					}
					Assert.fail("Cert with DN " + reqTrustedDN + " found but its alias is " 
							+ alias + " and not " + labelPrefix);
				}
			}
			if (num != casNum)
				Assert.assertEquals("Wrong number of CA certs in ca: ", casNum, num);
			if (!found)
				Assert.fail("Cert with DN " + reqTrustedDN + " not found.");
		} catch (Exception e)
		{
			e.printStackTrace();
			Assert.fail(e.toString());
		}
	}
}
