package de.fzj.unicore.ucc.actions;

import javax.xml.namespace.QName;

import org.apache.commons.cli.OptionBuilder;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.OperatingSystemType;
import org.unigrids.x2006.x04.services.tsf.TargetSystemFactoryPropertiesDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.TargetSystemFactory;
import de.fzj.unicore.uas.client.TSFClient;
import de.fzj.unicore.ucc.IServiceInfoProvider;
import de.fzj.unicore.ucc.helpers.Connector;
import eu.unicore.security.util.client.IClientProperties;

/**
 * "Connect" user to the sites. Will create TSSs if none found
 * 
 * @author schuller
 */
public class Connect extends ActionBase implements IServiceInfoProvider {

	/**
	 * the initial lifetime (in days) for newly created TSSs
	 */
	private int initialLifeTime;

	private static String lastRegistryURL;
	
	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_LIFETIME_LONG)
				.withDescription("Initial lifetime (in days) for created target systems.")
				.withArgName("Lifetime")
				.hasArg()
				.isRequired(false)
				.create(OPT_LIFETIME)
		);
	}

	@Override
	public String getName(){
		return "connect";
	}

	@Override
	public String getArgumentList(){
		return "";
	}

	@Override
	public String getSynopsis(){
		return "Connects to UNICORE. " +
		"If not yet done, target system services are initialised.";
	}
	@Override
	public String getDescription(){
		return "connect to UNICORE";
	}

	@Override
	public void process() {
		super.process();

		lastRegistryURL=registryURL;
		
		initialLifeTime=getNumericOption(OPT_LIFETIME_LONG, OPT_LIFETIME, -1);
		if(initialLifeTime>0){
			verbose("New TSSs will have a lifetime of <"+initialLifeTime+"> days.");
		}else{
			verbose("Using site default for TSS lifetime.");
		}

		Connector c=new Connector(registry, securityProperties, this);
		String black=properties.getProperty("blacklist", null);
		c.setBlacklist(black);
		c.run();
		int tsfAvailable=c.getAvailableTSF();
		if(tsfAvailable==0)message("There are no target system factories in the selected registry.");
		int tssAvailable=c.getAvailableTSS();
		message("You can access "+tssAvailable+" target system(s).");
		//it should be considered an error if no sites are available
		if(tssAvailable==0)endProcessing(ERROR);
	}

	@Override
	public String getCommandGroup(){
		return "General";
	}

	public QName getQName(){
		return TargetSystemFactory.TSF_PORT;
	}

	public String getServiceName(){
		return "TargetSystemFactory";
	}

	public String getServiceDetails(EndpointReferenceType epr, IClientProperties securityProperties){
		try{
			TSFClient tsf=new TSFClient(epr.getAddress().getStringValue(),epr,securityProperties);
			StringBuilder sb=new StringBuilder();
			try{
				int cpus=getTotalCPUs(tsf.getResourcePropertiesDocument());
				if(cpus>0)sb.append(cpus).append(" CPUs");
			}catch(Exception ex){}
			try{
				OperatingSystemType[]os=tsf.getOperatingSystems();
				if(os.length>0)sb.append(" ( ");
				for(OperatingSystemType o: os){
					sb.append(o.getOperatingSystemType().getOperatingSystemName().toString()).append(" ");
				}
				if(os.length>0)sb.append(")");
			}catch(Exception ex){}
			return sb.toString();
		}catch(Exception ex){}
		return null;
	}
	
	private int getTotalCPUs(TargetSystemFactoryPropertiesDocument rpDoc){
		int cpus=-1;
		if(rpDoc.getTargetSystemFactoryProperties().getTotalCPUCount()!=null){
			cpus=(int)rpDoc.getTargetSystemFactoryProperties().getTotalCPUCount().getRangeArray()[0].getUpperBound().getDoubleValue();
		}
		else if(rpDoc.getTargetSystemFactoryProperties().getIndividualCPUCount()!=null){
			cpus=(int)rpDoc.getTargetSystemFactoryProperties().getIndividualCPUCount().getRangeArray()[0].getUpperBound().getDoubleValue();
			if(rpDoc.getTargetSystemFactoryProperties().getTotalResourceCount()!=null){
				int nodes=(int)rpDoc.getTargetSystemFactoryProperties().getTotalResourceCount().getRangeArray()[0].getUpperBound().getDoubleValue();
				cpus*=nodes;
			}
		} 
		return  cpus;
	}
	
	static String getLastReqistryURL(){
		return lastRegistryURL;
	}
}
