package de.fzj.unicore.ucc.helpers;

import java.util.Calendar;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.atomic.AtomicInteger;

import org.oasisOpen.docs.wsrf.rl2.TerminationTimeDocument.TerminationTime;
import org.unigrids.x2006.x04.services.tsf.CreateTSRDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.TSFClient;
import de.fzj.unicore.uas.client.TSSClient;
import de.fzj.unicore.ucc.Command;
import de.fzj.unicore.ucc.InaccessibleServiceException;
import de.fzj.unicore.ucc.MessageWriter;
import de.fzj.unicore.wsrflite.xmlbeans.client.IRegistryQuery;
import eu.unicore.security.util.client.IClientProperties;

/**
 * connects to the Grid and creates target systems if required
 * 
 * @author schuller
 */
public class Connector implements Runnable {

	private final IRegistryQuery registry;
	
	private final IClientProperties securityProperties;

	private final MessageWriter msg;

	final AtomicInteger tsfAvailable=new AtomicInteger(0);
	final AtomicInteger tssAvailable=new AtomicInteger(0);

	/**
	 * space separated blacklist of sites to avoid
	 */
	private String blackList;
	
	/**
	 * the initial lifetime (in days) for newly created TSSs
	 */
	private int initialLifeTime;

	/**
	 * @param registry
	 * @param properties
	 * @param msg
	 */
	public Connector(IRegistryQuery registry, IClientProperties properties,	MessageWriter msg){
		this.registry=registry;
		this.securityProperties=properties;
		this.msg=msg;
	}
	
	@Override
	public void run() {

		TargetSystemFactoryLister lister=new TargetSystemFactoryLister(registry,securityProperties,msg);
		if(blackList!=null){
			msg.verbose("Using blacklist <"+blackList+">");
			String[] blacklist=blackList.trim().split(" ");
			lister.setBlacklist(blacklist);
		}
		GridTask<Boolean>task=new GridTask<Boolean>(){

			public TSFClient createClient(GridTaskTarget<Boolean> worker) throws Exception{
				EndpointReferenceType epr=worker.getEpr();
				return new TSFClient(epr.getAddress().getStringValue(),epr,securityProperties);
			}

			public Boolean run(GridTaskTarget<Boolean> worker)throws Exception{
				msg.verbose("Connecting to "+worker.getEpr().getAddress().getStringValue());
				TSFClient tsf=createClient(worker);
				tsfAvailable.incrementAndGet();
				List<EndpointReferenceType> eprs=tsf.getAccessibleTargetSystems();
				if(eprs==null || eprs.size()==0){
					try{
						CreateTSRDocument in=CreateTSRDocument.Factory.newInstance();
						in.addNewCreateTSR();
						if(initialLifeTime>0){
							TerminationTime tt=TerminationTime.Factory.newInstance();
							tt.setCalendarValue(getTermTime());
							in.getCreateTSR().setTerminationTime(tt);
						}
						TSSClient tss= tsf.createTSS(in);
						if(tss!=null){
							msg.verbose("Created TSS <"+tss.getTargetSystemName()+"> at address "+tss.getEPR().getAddress().getStringValue());
							tssAvailable.incrementAndGet();
						}
					}catch(Exception e){
						if(Command.getDetailMessage(e).contains("Access denied")){
							msg.verbose("Access denied on <"+tsf.getEPR().getAddress().getStringValue()+">");
						}
						else{
							msg.error("Can't create target system.",e);
						}
					}
				}
				else{
					tssAvailable.addAndGet(eprs.size());
				}
				return Boolean.TRUE;
			}
		};
		try{
			List<Future<Boolean>>results=lister.runOnEachTSF(task);
			//synchronize
			for(Future<Boolean> f: results){
				try{
					f.get();
				}catch(ExecutionException ee){
					if(! (ee.getCause() instanceof InaccessibleServiceException)){
						String message="Error accessing target system factory.";
						msg.error(message,ee.getCause());
					}
				}catch(Exception ex){
					msg.error("Error.", ex);
				}
			}
		}catch(Exception ex){
			msg.error("Error setting up connect tasks.", ex);
		}

	}

	protected Calendar getTermTime(){
		Calendar c = Calendar.getInstance();
		c.add(Calendar.DATE, initialLifeTime);
		return c;
	}
	
	/**
	 * sets the initial lifetime of new Target Systems
	 * @param days - lifetime in days
	 */
	public void setInitialLifetime(int days){
		if(days<0)throw new IllegalArgumentException();
		initialLifeTime=days;
	}
	
	public void setBlacklist(String blackList){
		this.blackList=blackList;
	}

	public int getAvailableTSS(){
		return tssAvailable.get();
	}
	
	public int getAvailableTSF(){
		return tsfAvailable.get();
	}

}
