package de.fzj.unicore.ucc.helpers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.TargetSystemFactory;
import de.fzj.unicore.uas.client.TSFClient;
import de.fzj.unicore.ucc.InaccessibleServiceException;
import de.fzj.unicore.ucc.MessageWriter;
import de.fzj.unicore.wsrflite.xmlbeans.client.IRegistryQuery;
import de.fzj.unicore.wsrflite.xmlbeans.client.RegistryClient;
import de.fzj.unicore.wsrflite.xmlbeans.client.Resources;
import eu.unicore.security.util.client.IClientProperties;

/**
 * Provides access to the target system factories in the registry.<br/>
 * 
 * The #list() and {@link #iterator()} methods will run over all the available
 * and accessible TSF services. If you need your own availability checks, and/or you
 * want to perform a task on each TSF, use the {@link #runOnEachTSF(GridTask)} method,
 * providing a custom {@link GridTask}.
 * 
 * @see GridTask
 * @see RegistryClient#listAccessibleServices(javax.xml.namespace.QName)
 * @author schuller
 */
public class TargetSystemFactoryLister implements Iterable<TSFClient>{

	private final IRegistryQuery registry;
	
	private final IClientProperties sec;
	
	private String[] blacklistPatterns=new String[0];
	
	private int runningLimit=10;
	
	private final MessageWriter msg;
	
	public TargetSystemFactoryLister(IRegistryQuery registry, IClientProperties sec, MessageWriter msg){
		this.registry=registry;
		this.sec=sec;
		this.msg=msg;
	}
	
	public void setBlacklist(String[] blacklist){
		this.blacklistPatterns=blacklist;
	}
	/**
	 * return a list of all accessible target system factories, 
	 * providing a pre-initialised TSFClient per target system factory. 
	 * A black list will be honored.
	 * @return
	 */
	public List<TSFClient> list(){
		List<TSFClient> tss=new ArrayList<TSFClient>();
		try{
			List<Future<TSFClient>> tasks=runOnEachTSF(new GetTSFClientTask());
			while(tasks.size()>0){
				Future<TSFClient>task=tasks.get(0);
				try{
					TSFClient res=task.get(500, TimeUnit.MILLISECONDS);
					if(res!=null){
						tss.add(res);
						tasks.remove(0);
					}
				}catch(TimeoutException te){/*no worries will retry*/}
				catch(ExecutionException ee){
					if(! (ee.getCause() instanceof InaccessibleServiceException)){
						String message="Error accessing target system factory.";
						msg.error(message,ee.getCause());
					}
					tasks.remove(0);
				}			
			}
		}catch(Exception e){
			String message="Can't create list of target systems.";
			msg.error(message,e);
		}
		return tss;
	}
	
	/**
	 * return an iterator over the available target system factories, providing a
	 * pre-initialised TSFClient per instance
	 * <br/>
	 * in case of errors, an empty iterator is returned
	 * 
	 * @see de.fzj.unicore.uas.client.TSFClient
	 */
	public Iterator<TSFClient> iterator(){
		return list().iterator();
	}

	/**
	 * asynchronously execute a {@link GridTask} on each TSF service.
	 * <br/>It will honor a possible blacklist.
	 * @see GridTaskTarget
	 * @return a list of {@link Future} classes for getting the results of the tasks
	 */
	public <T> List<Future<T>>runOnEachTSF(GridTask<T>task)throws Exception{
		List<Future<T>>result=new ArrayList<Future<T>>();
		final List<EndpointReferenceType> tsfs=registry.listServices(TargetSystemFactory.TSF_PORT);
		for(EndpointReferenceType epr: tsfs){
			if(isBlacklisted(epr)){
				msg.verbose("Ignoring blacklisted site "+epr.getAddress().getStringValue());
				continue;
			}
			GridTaskTarget<T> worker=new GridTaskTarget<T>(sec,registry,msg);
			worker.setEpr(epr);
			worker.setTask(task);
			
			boolean submittedTask;
			do{
				submittedTask=false;
				int running=GridTaskTarget.getRunningTasks();
				if(running<runningLimit){
					result.add(Resources.getExecutorService().submit(worker));
					submittedTask=true;
				}else{
					Thread.sleep(500);
				}
			}while(!submittedTask);
			
		}
		return result;
	}
	
	protected boolean isBlacklisted(EndpointReferenceType epr){
		boolean result=false;
		String url=epr.getAddress().getStringValue();
		for(String pattern: blacklistPatterns){
			if(url.contains(pattern))return true;
		}
		return result;
	}
	
	/**
	 * simple GridTask that just creates a TSFClient for each target epr and checks
	 * its responsiveness and accessibility by calling getCurrentTime()
	 */
	public static class GetTSFClientTask implements GridTask<TSFClient>{

		private TSFClient createClient(GridTaskTarget<TSFClient> worker)throws Exception {
			EndpointReferenceType epr=worker.getEpr();
			return new TSFClient(epr.getAddress().getStringValue(),epr,worker.getSecurity());
		}

		public TSFClient run(GridTaskTarget<TSFClient> target) throws Exception {
			TSFClient client=createClient(target);
			client.getCurrentTime();
			return client;
		}
		
	}
	
}
