/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 23, 2011
 * Author: K. Stasiak <karol.m.stasiak@gmail.com>
 */
package de.fzj.unicore.ucc.helpers;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.util.Enumeration;
import java.util.regex.Pattern;

import javax.net.ssl.SSLContext;

import org.apache.log4j.Logger;

import pl.edu.icm.unicore.uvos.util.Configuration;
import eu.unicore.security.util.client.IAuthenticationConfiguration;
import eu.unicore.security.util.client.IClientProperties;

/**
 * Helper class containing configuration for SAML VO query client.
 * @author kstasiak
 */
public class VOConfiguration implements IAuthenticationConfiguration {

	private static final Logger logger=Logger.getLogger(VOConfiguration.class.getCanonicalName());

	protected Configuration conf;
	protected String cfgFn;
	protected IClientProperties securityProperties;
	protected File propertiesFile;
	protected final String voUrlAsString;

	private final static String URL_PARAM_CHARACTER = "[a-zA-Z0-9_\\+%\\.~\\-]";
	private final static String URL_WORD = "[a-zA-Z0-9~_\\-]+";
	private final static String URL_FILE = "[a-zA-Z0-9\\.~_\\-]*";
	
	private static final Pattern URL_REGEX = Pattern.compile(
			("https?://"
			+ "(W\\.)*W"
			+ ":[1-9][0-9]*"
			+ "(/F)*"
			+ "(\\?"
				+ "("
					+ "(P+(=P*)?&|&)*"
					+ "P+(=P*)?&?"
				+ ")?"
			+ ")?(\\#.*)?")
			.replaceAll("P", URL_PARAM_CHARACTER)
			.replaceAll("F", URL_FILE)
			.replaceAll("W", URL_WORD));
	
	public static boolean isValidUrl(String url){
		return URL_REGEX.matcher(url).matches();
	}

	public VOConfiguration(File propertiesFile, IClientProperties securityProperties, String voUrl) {
		this.securityProperties = securityProperties;
		this.propertiesFile = propertiesFile;
		this.voUrlAsString=voUrl;
		if (!isValidUrl(voUrl)) {
			logger.error("URL '"+voUrl+"' does not look valid. "
					+ "The correct format is (http|https)://HOST:PORT");
		}
		try {
			conf = new Configuration(propertiesFile.getAbsolutePath());
		} catch (Exception ex) {
			logger.error("Unexpected error when loading configuration file: "
					+ propertiesFile.getAbsolutePath(),
					ex);
		}
	}

	public int getServerPort() {
		try {
			return Integer.parseInt(voUrlAsString.split(":")[2].split("/")[0]);
		}
		catch (RuntimeException ex){
			logger.error("URL '" + voUrlAsString + "' doesn't contain valid port number.");
			throw ex;
		}
	}

	public String getServerHost() {
		try {
			return voUrlAsString.split(":")[1].split("/")[2];
		}
		catch (RuntimeException ex){
			logger.error("URL '" + voUrlAsString + "' doesn't contain server name.");
			throw ex;
		}
	}

	public String getProtocol() {
		return voUrlAsString.split(":")[0];
	}

	public String getServicePath() {
		return "";
	}

	@Override
	public boolean doHttpAuthn() {
		return false;
	}

	@Override
	public String getHttpUser() {
		return null;
	}

	@Override
	public String getHttpPassword() {
		return null;
	}

	@Override
	public boolean doSSLAuthn() {
		return true;
	}

	@Override
	public String getKeystore() {
		return securityProperties.getKeystore();
	}

	@Override
	public String getKeystorePassword() {
		return securityProperties.getKeystorePassword();
	}

	private String cachedAlias;
	private boolean isAliasCached=false;
	@Override
	public String getKeystoreAlias() {
		if(isAliasCached) return cachedAlias;
		cachedAlias = securityProperties.getKeystoreAlias();
		if(cachedAlias!=null) {
			isAliasCached=true;
			return cachedAlias;
		}
		try {
			KeyStore instance = KeyStore.getInstance(securityProperties.getKeystoreType());
			instance.load(new FileInputStream(securityProperties.getKeystore()), securityProperties.getKeystorePassword().toCharArray());
			Enumeration<String> e = instance.aliases();
			for(; e.hasMoreElements() ;) {
				String i = e.nextElement();
				if(instance.isKeyEntry(i)){
					if(cachedAlias==null){
						cachedAlias=i;
						isAliasCached=true;
					}
					else{
						cachedAlias=null;
						isAliasCached=true;
						logger.error("Found multiple keys in the keystore");
						break;
					}
				}
			}
			if(cachedAlias!=null){
				logger.info("Found one private key entry under alias \'"+cachedAlias+"\'");
			}
			else{
				logger.error("Could not determine private key alias.");
			}
		} catch (IOException ex) {
			logger.error("Cannot read keystore: "+securityProperties.getKeystore(), ex);
		} catch (NoSuchAlgorithmException ex) {
			logger.error("Cannot read keystore: "+securityProperties.getKeystore(), ex);
		} catch (CertificateException ex) {
			logger.error("Cannot read keystore: "+securityProperties.getKeystore(), ex);
		} catch (KeyStoreException ex) {
			logger.error("Cannot read keystore: "+securityProperties.getKeystore(), ex);
		}	
		return cachedAlias;
	}

	@Override
	public String getKeystoreKeyPassword() {
		return securityProperties.getKeystoreKeyPassword();
	}


	@Override
	public String getKeystoreType() {
		return securityProperties.getKeystoreType();
	}

	@Override
	public String getTruststore() {
		return securityProperties.getTruststore();
	}

	@Override
	public String getTruststorePassword() {
		return securityProperties.getTruststorePassword();
	}

	@Override
	public String getTruststoreType() {
		return securityProperties.getTruststoreType();
	}

	/**
	 * Makes shallow copy of this configuration.
	 * @return
	 */
	@Override
	public IAuthenticationConfiguration clone() {
		return new VOConfiguration(propertiesFile, securityProperties, voUrlAsString);
	}

	@Override
	public SSLContext getSSLContext() {
		return securityProperties.getSSLContext();
	}

	public Configuration getConfiguration() {
		return conf;
	}

	public IClientProperties getSecurityProperties() {
		return securityProperties;
	}
}
