package de.fzj.unicore.ucc.workflow;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.NumberFormat;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.chemomentum.common.impl.workflow.WorkflowManagementClient;
import org.chemomentum.common.ws.ITrace;
import org.chemomentum.tracer.xmlbeans.EntryDocument.Entry;
import org.chemomentum.tracer.xmlbeans.FilterEntriesRequestDocument;
import org.chemomentum.tracer.xmlbeans.FilterEntriesRequestDocument.FilterEntriesRequest;
import org.chemomentum.tracer.xmlbeans.FilterEntriesResponseDocument;
import org.chemomentum.workflow.xmlbeans.WorkflowResourcePropertiesDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.ucc.actions.ActionBase;
import de.fzj.unicore.wsrflite.xmlbeans.WSUtilities;
import eu.unicore.security.xfireutil.client.SecuredXFireClientFactory;

/**
 * retrieve workflow trace info (and do some analysis on it)
 * 
 * TODO this is probably rather outdated and needs to be synched with 
 * the actual contents of the Tracer service
 * 
 * @author schuller
 */
public class WorkflowTrace extends ActionBase {
	
	private EndpointReferenceType workflowEPR;
	private String workflowID;
	private WorkflowManagementClient workflow;
	private FilterEntriesResponseDocument traceEntries;
	private String OPERATION_SUBMIT_WORKFLOW="submitWorkflow";
	private String OPERATION_SUBMITWA="submitWorkAssignment";
	private String OPERATION_SUBMIT="Submit";
	private String OPERATION_FINISHEDWA="finishedWA";
	private String OPERATION_FINISHEDJOB="finishedJob";
	private String OPERATION_FAILEDWA="failedWA";
	
	final NumberFormat outputFormat=NumberFormat.getNumberInstance();
	
	@Override
	public String getName() {
		return "workflow-trace";
	}
	
	@Override
	public String getArgumentList(){
		return "[<workflow-address> | <workflow-file>]";
	}
	
	@Override
	public String getSynopsis(){
		return "Retrieves trace info about a workflow. " +
				"The workflow address is given on the command line or is " +
				"read from <workflow-file>. " +
				"The output shows the mean time taken to process " +
				"jobs as well as latencies in the workflow engine, i.e. the " +
				"time between workflow submission and submission of tasks to the " +
				"service orchestrator.";
	}

	@Override
	public String getDescription(){
		return "trace info on a workflow.";
	}
	@Override
	public String getCommandGroup(){
		return "Workflow";
	}
	
	@Override
	public void process(){
		super.process();
		try{
			run();
		}catch(Exception e){
			error("",e);
			endProcessing(1);
		}
	}

	//read the workflow address
	protected void initEPR()throws Exception{
		String arg=null;
		if(getCommandLine().getArgs().length>1)arg=getCommandLine().getArgs()[1];
		
		if(arg==null){
			try{
				arg=new BufferedReader(new InputStreamReader(System.in)).readLine();
			}catch(Exception e){
				error("Can't read workflow address from stdin.",e);
				endProcessing(1);
			}
		}
		
		File wf=new File(arg);
		if(wf.exists())
		{
			workflowEPR=EndpointReferenceType.Factory.parse(wf);
			verbose("Read Workflow EPR from <"+arg+">");
		}
		else{
			try{
				new URL(arg);
			}
			catch(MalformedURLException me){
				error("The argument <"+arg+"> is not a valid URL.",null);
				endProcessing(1);
			}
			//arg is an Address
			workflowEPR=EndpointReferenceType.Factory.newInstance();
			workflowEPR.addNewAddress().setStringValue(arg);
		}
		String url=workflowEPR.getAddress().getStringValue();
		verbose("Checking workflow at "+url);
		workflow=new WorkflowManagementClient(url,workflowEPR,securityProperties);
		workflowID=WSUtilities.extractResourceID(workflow.getEPR());
	}
	
	protected void run() throws Exception{
		outputFormat.setMaximumFractionDigits(2);
		outputFormat.setMinimumFractionDigits(2);
		
		initEPR();
		getTrace();
		writeToFile();
		analyseTrace();
	}

	protected void getTrace()throws Exception{
		EndpointReferenceType tracerEPR=null;
		
		try{
			WorkflowManagementClient wfmc=new WorkflowManagementClient(workflowEPR.getAddress().getStringValue(),workflowEPR,securityProperties);
			//TODO fix WorkflowManagementClient
			WorkflowResourcePropertiesDocument rpDoc=WorkflowResourcePropertiesDocument.Factory.parse(wfmc.GetResourcePropertyDocument().getGetResourcePropertyDocumentResponse().newInputStream());
			tracerEPR=rpDoc.getWorkflowResourceProperties().getTracerAddress();
		}
		catch(Exception ex){
			error("Error getting tracer address from workflow service.", ex);
			endProcessing(1);
		}
		if(tracerEPR==null){
			List<EndpointReferenceType>tracerEprs=registry.listServices(ITrace.PORT);
			//TODO check TracerAddress property on the workflow instead
			if(tracerEprs.size()<1){
				message("No tracer available!");
				endProcessing(1);
			}
			else if(tracerEprs.size()>1){
				verbose("Warning: multiple tracers avaible, using first one...");
			}
			tracerEPR=tracerEprs.get(0);
		}
		
		String tracerURL=tracerEPR.getAddress().getStringValue();
		verbose("Using tracer at <"+tracerURL+">");
		ITrace tracer=new SecuredXFireClientFactory(securityProperties).createPlainWSProxy(ITrace.class, tracerEPR.getAddress().getStringValue());
		
		FilterEntriesRequestDocument ferd=FilterEntriesRequestDocument.Factory.newInstance();
		FilterEntriesRequest fer=ferd.addNewFilterEntriesRequest();
		fer.setField(ITrace.MESSAGE_ID);
		fer.setPattern("%"+workflowID+"%");
		traceEntries=tracer.filterEntries(ferd);
		verbose("Trace read.");
	}
	
	
	protected void writeToFile(){
		try{
			File out=new File(output,workflowID+".trace");
			FileWriter fw=new FileWriter(out);
			fw.write(traceEntries.toString()+"\n");
			fw.flush();
			fw.close();
			verbose("Trace written to "+out.getAbsolutePath());
		}catch(Exception e){
			error("Could not write to file.",e);
			endProcessing(1);
		}
	}
	
	
	/**
	 * extract some info about the workflow.
	 * 
	 * Total number of workassignments
	 * Number of successful or failed WAs
	 * Mean time for processing each (successful) WA
	 * Total time for processing the workflow (approx)
	 * 
	 */
	protected void analyseTrace(){
		Map<String, WAStat>waInfo=new HashMap<String, WAStat>();
		Calendar start=null;
		Calendar firstWASent=null;
		Calendar lastWASent=null;
		Calendar end=null;
		int successfulWAs=0;
		int failedWAs=0;
		int totalWAs=0;
		float mean=0;
		float meanLatency=0;
		float meanTime=0;
		
		Entry[] entries=traceEntries.getFilterEntriesResponse().getTrace().getEntryArray();
		
		try{
			for(Entry entry:entries){
				String operation=entry.getOperation();
				Calendar thisTime=entry.getTimeStamp();
				
				if(OPERATION_SUBMIT_WORKFLOW.equalsIgnoreCase(operation)){
					start=thisTime;
					verbose("["+operation+"], time: "+thisTime.getTime().toString());
				}
				else{
					String id=null;
					String messageID=entry.getMessageId();
					if(messageID.startsWith("WA ID: ")){
						//TODO clumsy, should have uniform message IDs 
						id=messageID.substring(7);
					}
					else{
						StringBuilder sb=new StringBuilder();
						String[] s=messageID.split("/");
						for(int i=0;i<4;i++){
							if(i>0)sb.append("/");
							sb.append(s[i]);
						}
						id=sb.toString();
					}
					verbose("["+operation+"] id "+id+", time: "+thisTime.getTime().toString());

					if(start==null||start.after(thisTime)){
						start=thisTime;
					}
					if(end==null||end.before(thisTime)){
						end=thisTime;
					}

					WAStat w=waInfo.get(id);
					if(w==null){
						w=new WAStat();
						waInfo.put(id,w);
					}

					if(OPERATION_SUBMITWA.equalsIgnoreCase(operation)){
						w.start=thisTime;
						if(firstWASent==null||firstWASent.after(thisTime)){
							firstWASent=thisTime;
						}
						if(lastWASent==null||lastWASent.before(thisTime)){
							lastWASent=thisTime;
						}
					}

					if(OPERATION_SUBMIT.equalsIgnoreCase(operation)){
						w.submissionTime=thisTime;
					}

					if(OPERATION_FINISHEDJOB.equalsIgnoreCase(operation)){
						w.finishTime=thisTime;
					}

					if(OPERATION_FINISHEDWA.equalsIgnoreCase(operation)){
						w.end=thisTime;
						successfulWAs++;
						totalWAs++;
					}

					if(OPERATION_FAILEDWA.equalsIgnoreCase(operation)){
						w.end=thisTime;
						failedWAs++;
						totalWAs++;
					}
				}
			}
			int numTraceData=0;
			int numMissingData=0;
			//calculate stats
			for(String id:waInfo.keySet()){
				try{
					WAStat w = waInfo.get(id);
					//calculate wa processing time and re-compute mean value
					long time=w.end.getTimeInMillis()-w.start.getTimeInMillis();
					mean=mean+time;

					//calculate wa latency
					time=w.submissionTime.getTimeInMillis()-w.start.getTimeInMillis();
					meanLatency=meanLatency + time;
					verbose("Latency for "+id+": "+outputFormat.format(time/1000));

					//calculate job processing time
					time=w.finishTime.getTimeInMillis()-w.submissionTime.getTimeInMillis();
					meanTime=meanTime+time;
					verbose("Processing time for "+id+": "+outputFormat.format(time/1000));
					numTraceData++;
				}catch(Exception e){
					error("Error processing (or missing) trace data for "+id,e);
					numMissingData++;
				}
			}
			mean=mean/numTraceData;
			meanLatency=meanLatency/numTraceData;
			meanTime=meanTime/numTraceData;
			
			//print stats
			message("*** Statistics for "+workflowEPR.getAddress().getStringValue());
			message("");
			message("Total time: "+(end.getTimeInMillis()-start.getTimeInMillis())/1000+" seconds.");
			message("");
			message("Total number of workassignments: "+totalWAs);
			message("Successful: "+successfulWAs+" , failed: "+failedWAs);
			message("Mean time per workassignment: "+outputFormat.format(mean/1000)+" secs.");
			message("Mean workassignment latency: "+outputFormat.format(meanLatency/1000)+" secs.");
			message("Mean job processing time: "+outputFormat.format(meanTime/1000)+" secs.");
			long latencyForFirstWA=firstWASent.getTimeInMillis()-start.getTimeInMillis();
			long latencyForLastWA=lastWASent.getTimeInMillis()-start.getTimeInMillis();
			message("Workflow engine latency for first WA: "+outputFormat.format(latencyForFirstWA/1000)+" secs.");
			message("Workflow engine latency for last WA: "+outputFormat.format(latencyForLastWA/1000)+" secs.");
			
			message("");
			if(numMissingData>0){
				message("Missing trace data for "+numMissingData+" workassignments.");
			}
			message("");
			
		}catch(Exception e){
			error("Could not extract analysis data.",e);
		}

	} 
	
	private class WAStat{
		public Calendar start;
		public Calendar submissionTime;
		public Calendar finishTime;
		public Calendar end;
	}
	
}
